<?php

class Archive
{
    private $BP;
    private $dpuID;
    private $media;
    private $path;
    private $bReplace;

    function __construct($BPin){
        $this->BP = $BPin;
        $this->dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $this->BP->get_local_system_id();
        $this->storageList = array();
        $this->defaultDeviceName = "";
    }

    function setBReplace(){
        global $Common;
        $this->bReplace = $Common->getbReplace();
        if ($this->bReplace) {
            echo "You have chosen to use the new appliance's storage configuration.\n";
            if ($this->getStorageList() === false) {
                echo "\n" .
                    "There is no backup storage configured on the new appliance.\n" .
                    "You must re-start and either use the storage configuration of the original appliance,\n" .
                    "or define backup storage on this appliance, then restart disaster_recovery.\n";
                exit(-1);
            }
            //var_dump($this->storageList);
            //$storageNames = array_map(function($o) { return $o['name']; }, $this->storageList);
            //var_dump($storageNames);
        } else {
            echo "You have chosen to use the original appliance's storage configuration.\n";          
        }
    }
    
    function getStorageList() {
        $result = false;
        $storage = $this->BP->get_storage_list();
        if ($storage !== false) {
            foreach ($storage as $id => $name) {
                $info = $this->BP->rest_get_storage_info($id);
                if ($info !== false) {
                    if ($info['usage'] == 'stateless' || $info['usage'] == 'backup') {
                        $devices = $info['devices'];
                        if (count($devices) > 0) {
                            $deviceInfo = $this->BP->get_device_info($devices[0]);
                            $info['device_name'] = $deviceInfo !== false ? $deviceInfo['dev_name'] : "";
                            if ($info['default']) {
                                $this->defaultDeviceName = $info['device_name'];
                            }
                        }
                        $this->storageList[] = $info;
                        $result = true;
                    }
                }
            }
        } else {
            echo "Error getting storage configuration on appliance: " . $this->BP->getError();
        }
        return $result;
    }

    function getMediaList(){
        $daemonStatus = $this->BP->get_uarchive_status($this->dpuID);
        $result = NULL;
        if ($daemonStatus !== false) {
            if ($daemonStatus === "off") {
                $this->BP->start_uarchive($this->dpuID);
            }
            $result = $this->BP->get_connected_archive_media($this->dpuID);

        }

        return $result;
    }

    function mountMedia($mediaName){
        $this->media = $mediaName;
        $mediaInfo = $this->BP->get_connected_archive_media($this->dpuID);
        if ($mediaInfo !== false) {
            foreach ($mediaInfo as $media) {
                if (isset($media['type']) && $media['type'] == 'external') {
                    // If external media, we need to check the storage and connect if not connected.
                    $storageList = $this->BP->get_storage_list($this->dpuID);
                    if ($storageList !== false) {
                        foreach ($storageList as $id => $name) {
                            if ($mediaName == $name) {
                                $result = $this->BP->connect_storage($id, $this->dpuID);
                                break;
                            }
                        }
                    }
                }
            }
        }
        if ($result !== false) {
            $result = $this->BP->mount_archive_media($this->media, $this->dpuID);           
        }
        if ($result == false) {
            echo "Error: mounting media " . $mediaName . ": " . $this->BP->getError() . "\n";
        } else {
            echo "The media has been successfully mounted.\n";
        }
        return $result;
    }

    function findPath(){
        $result = true;
        $this->path = $this->BP->get_archive_state_path($this->media, $this->dpuID);
        if ($this->path !== false){
            $mode = $this->BP->get_dr_from_archive_mode($this->path, $this->dpuID);
            if ($mode == false) {
                echo "Error: could not locate recovery information on appliance metadata: " . $this->BP->getError() . "\n";
                $result = false;
            }
        } else {
            echo "Error: could not find the path or appliance metadata on selected backup copy media: " . $this->BP->getError() . "\n" .
                "Please run disaster_recovery again.\n";
            $result = false;
        }
        return $result;
    }

    function metadataCheck(){

        $path = $this->path;
        $bReplace = $this->bReplace;
        $action = 'check';

        $filter = array('backup_path' => $path,
            'save_new_storage' => $bReplace,
            'action' => $action);

        $result = $this->BP->restore_appliance_metadata($filter);
        if ($result !== false){
            if (!empty($result)){
                global $Log;
                foreach($result as $msg) {
                    $Log->writeVariable($msg);
                }
                $result = array();
            }
        } else {
            echo "Error checking appliance metadata for recovery: " . $this->BP->getError() . "\n";
        }
        return $result;
    }

    function metadataRestore(){
        $path = $this->path;
        $bReplace = $this->bReplace;
        $action = 'restore';

        $filter = array('backup_path' => $path,
            'save_new_storage' => $bReplace,
            'action' => $action);

        //we have to cancel any running jobs before restoring system metadata
        $jobList = $this->BP->get_job_list($this->dpuID);

        if ($jobList !== false) {
            foreach ($jobList as $id => $backup) {
                $this->BP->cancel_job($id);
            }
        }

        $result = $this->BP->restore_appliance_metadata($filter);


        if ($result !== false){
            return true;
        } else {
            echo $this->BP->getError();
            return false;
        }
    }

    function isEncryptionEnabled(){
        $result = $this->BP->is_archive_dpu_encryption_enabled($this->path);
        if ($result !== false){
            return true;
        } else {
            return false;
        }
    }

    function restoreState(){
        $selectedMedia = $this->media;
        $path = $this->BP->get_archive_state_path($selectedMedia, $this->dpuID);
        if ($path !== false) {
            $versionCheck = $this->BP->dpu_restore_archive_check_version($path);
            if($versionCheck !== false) {
                $result = $this->BP->dpu_restore_archive_state($selectedMedia, $path);
                if ($result !== false) {
                    return true;
                } else {
                    echo $this->BP->getError();
                }
            } else {
                echo $this->BP->getError();
            }
        }else {
            echo $this->BP->getError();
        }
    }

    function getClients(){
        global $Log;
        $clientList = array();
        $filter = array();
        $filter['target'] = $this->media;

        $result = $this->BP->import_archive_catalog($this->media, "", false, $this->dpuID);
        if ($result !== false) {

            $archiveList = $this->BP->get_archive_status($filter);

            if($archiveList !== false){
                foreach($archiveList as $archive){

                    $clientPresent = in_array($archive['client_name'], $clientList);
                    if(!$clientPresent) {
                        $clientList[] = $archive['client_name'];
                    }
                }
            }
        }
        return $clientList;
    }

    function validate($clients, $bTarget){
        $restoreClientDevices = $clients;

        $devices = false;
        if($restoreClientDevices !== false) {
            foreach($restoreClientDevices as $ClientDevices){
                if (array_key_exists('device_name', $ClientDevices)) {
                    $devices[] = $ClientDevices['device_name'];
                }
            }
            if($bTarget === false) {
                $devices = array();
            }
            $validate = $this->BP->archive_validate_devices($bTarget, $devices);

            if($validate !== false){
                return true;
            }else {
                return false;
            }
        }else {
            return false;
        }
    }

    function createClients($line){
        if (!is_array($line)) {
            $line = array($line);
        }

        $clientList = array();
        foreach ($line as $client) {
            $clientArray = array('client_name' => $client);
            $clientList[] = $clientArray;
        }

        // If they chose bReplace and have > 1 storage, add the device_name for the restore.
        if ($this->bReplace) {
            $this->setStorageForClients($clientList);
            //var_dump($clientList);
        }
        
        return $clientList;
    }

    /*
     * If the user is using the new appliance storage and there is > 1 backup storage, see which one
     * they want to use for each client.  Otherwise, the default device will be used.
     * Modifies the client list in place.
     */
    function setStorageForClients(&$clientList) {
        // Initialize the client array to the default device.
        foreach ($clientList as &$client) {
            $client['device_name'] = $this->defaultDeviceName;
        }

        // If there is more than one backup storage, let the user choose it, per client.
        if (count($this->storageList) > 1) {
            $storageNames = array_map(function($o) { return $o['name']; }, $this->storageList);
            foreach ($clientList as &$client) {
                // Ask the user to select from the storage names.
                $selectedStorage = createMenu("Select Storage for " . $client['client_name'] . ":", $storageNames);
                foreach ($this->storageList as $info) {
                    if ($info['name'] == $selectedStorage) {
                        $client['device_name'] = $info['device_name'];
                        break;
                    }
                }
            }
        }
    }

    function preRestore($restoreClientListAll){
        global $Log;
        $restoreClientList = false;
        if($restoreClientListAll !== false){
            $restoreClientList = array();
            foreach($restoreClientListAll as $restoreClient){
                $clientPresent = in_array($restoreClient['client_name'], $restoreClientList);
                if(!$clientPresent) {
                    $restoreClientList[] = $restoreClient['client_name'];
                    if($this->bReplace){
                        $deviceName[] = $restoreClient['device_name'];
                    }
                }
            }
        }

        $setImported = false;
        $failureCount = 0;
        $filter = array();
        while (!$setImported && $failureCount < 10) {
            $archive_sets = $this->BP->get_archive_sets($filter, $this->dpuID);
            foreach ($archive_sets as $set) {
                if (strcmp($set['status'], "import success") == 0) {
                    $Log->writeVariable(count($archive_sets));
                    $Log->writeVariable("DR Actual import of the archive set ". $set['archive_set_id']. " succeeded.");
                    $setImported = true;
                    continue;
                } else if(strcmp($set['status'], "import failed") == 0) {
                    $Log->writeVariable("DR Actual import of the archive set ". $set['archive_set_id']. " failed.");
                    $setImported = false;
                    $failureCount++;
                } else if(strcmp($set['status'], "import in progress") == 0) {
                    $Log->writeVariable("DR Actual import in progress for set ". $set['archive_set_id']. ".");
                    $setImported = false;
                } else {
                    $setImported = false;
                    $failureCount++;
                }
                sleep(2);
            }
            sleep(15);
        }
        $Log->writeVariable('DR done with import');

        // temporary - lmc, pass in from RRC.
        // DPU is special, used for local directory archives.  We need to make that clear.
        if ($restoreClientList !== false || $restoreClientList !== NULL) {
            $archiveMap = $this->getClientMap($this->media);
            if ($archiveMap !== false || $archiveMap !== NULL){
                $this->gatherAndRestoreArchives($restoreClientList, $archiveMap);
            } else {
                echo "Error, exiting.";
            }
        } else {
            echo "Error, exiting.";
        }
    }

    function getClientMap(){
        $media = $this->media;
        $BP = $this->BP;
        global $Log;
        $archiveMap = array();
        $filter['target'] = $media;
        $archiveList = $BP->get_archive_status($filter);
        if ($archiveList !== false) {
            foreach($archiveList as $archive) {
                $clientPresent = isset($archiveMap[$archive['client_name']]);
                //$clientPresent = in_array($archive['client_name'], $archiveMap);
                if($clientPresent) {
                    $aList = $archiveMap[$archive['client_name']];
                    $aList[] = $archive;
                    $archiveMap[$archive['client_name']] = $aList;

                } else {

                    $aList = array();
                    $aList[] = $archive;
                    $archiveMap[$archive['client_name']] = $aList;
                }
            }
        }

        return $archiveMap;
    }

    function gatherAndRestoreArchives($restoreClientList, $archiveMap){
        $media = $this->media;
        $BP = $this->BP;
        global $Log;

        $Log->writeVariable("DR in gather and restore");
        foreach($restoreClientList as $client) {
            $instanceList = array();

            //echo "In client: $client\n";

            if (!isset($archiveMap[$client])) {
                $Log->writeVariable('Skip client');
                $Log->writeVariable($client);
                echo "Skip client: $client\n";
                continue;
            }
            $archiveList = $archiveMap[$client];
            usort($archiveList, "compare_date");
            $mastIncrList = array();
            $selectiveList = array();
            $sql = array();
            $exchange = array();
            $vm = array();
            $hyperv = array();
            $xen = array();
            $oracle = array();
            $sharepoint = array();
            $exchangeList = array();
            $sqlList = array();
            $hypervList = array();
            $vmList = array();
            $oracleList = array();
            $sharepointList = array();
            $xenList = array();
            $bareMetalList = array(); // will always be of size 1
            unset($localDirRestore);
            unset($CEPRestore);

            foreach ($archiveList as $archive) {
                $Log->writeVariable("DR in archive list loop, dumping archive....");
                $Log->writeVariable($archive);

                if ($archive['success'] == 0) {
                    $Log->writeVariable($archive);
                    continue;
                }
                // group backup types to restore and restore only each of the last group per instance
                if ($archive['type'] == "baremetal") {
                    $bareMetalList = array();
                    $bareMetalList[] = $archive;
                } else if ($archive['type'] == "selective") {
                    $selectiveList[] = $archive;
                } else if ($archive['type'] == "CEP") {
                    $CEPRestore = $archive;
                } else if ($archive['type'] == "localdir") {
                    $localDirRestore = $archive; // check if this is correct?
                } else if ($archive['type'] == "master") {
                    $mastIncrList = array();
                    $mastIncrList[] = $archive;
                } else if ($archive['type'] == "differential") {
                    $mastIncrList[] = $archive;
                } else if ($archive['type'] == "exchange full") {
                    $exchangeList = array();
                    $exchangeList[] = $archive;
                    $exchange[$archive['instance_id']] = $exchangeList;
                } else if ($archive['type'] == "exchange differential" || $archive['type'] == "exchange incremental") {
                    $exchangeList = $exchange[$archive['instance_id']];
                    $exchangeList[] = $archive;
                    $exchange[$archive['instance_id']] = $exchangeList;
                } else if ($archive['type'] == "mssql full") {
                    $sqlList = array();
                    $sqlList[] = $archive;
                    $sql[$archive['instance_id']] = $sqlList;
                } else if ($archive['type'] == "mssql differential" || $archive['type'] == "mssql incremental") {
                    $sqlList = $sql[$archive['instance_id']];
                    $sqlList[] = $archive;
                    $sql[$archive['instance_id']] = $sqlList;
                } else if ($archive['type'] == "vmware full") {
                    $vmList = array();
                    $vmList[] = $archive;
                    $vm[$archive['instance_id']] = $vmList;
                } else if ($archive['type'] == "vmware differential" || $archive['type'] == "vmware incremental") {
                    $vmList = $vm[$archive['instance_id']];
                    $vmList[] = $archive;
                    $vm[$archive['instance_id']] = $vmList;
                } else if ($archive['type'] == "hyperv full") {
                    $hypervList = array();
                    $hypervList[] = $archive;
                    $hyperv[$archive['instance_id']] = $hypervList;
                } else if ($archive['type'] == "hyperv incremental") {
                    $hypervList = $hyperv[$archive['instance_id']];
                    $hypervList[] = $archive;
                    $hyperv[$archive['instance_id']] = $hypervList;
                } else if ($archive['type'] == "oracle full") {
                    $oracleList = array();
                    $oracleList[] = $archive;
                    $oracle[$archive['instance_id']] = $oracleList;
                } else if ($archive['type'] == "oracle incr") {
                    $oracleList = $oracle[$archive['instance_id']];
                    $oracleList[] = $archive;
                    $oracle[$archive['instance_id']] = $oracleList;
                } else if ($archive['type'] == "sharepoint full") {
                    $sharepointList = array();
                    $sharepointList[] = $archive;
                    $sharepoint[$archive['instance_id']] = $sharepointList;
                } else if ($archive['type'] == "sharepoint diff") {
                    $sharepointList = $sharepoint[$archive['instance_id']];
                    $sharepointList[] = $archive;
                    $sharepoint[$archive['instance_id']] = $sharepointList;
                } else if ($archive['type'] == "xen full") {
                    $xenList = $xen[$archive['instance_id']];
                    $xenList[] = $archive;
                    $xen[$archive['instance_id']] = $xenList;
                }

            }

            // call restore_archive here for all backup types of a client
            if (count($bareMetalList)) {
                foreach ($bareMetalList as $key => $ar) {
                    $this->restore_archive($ar);
                }
            }
            if (count($selectiveList)) {
                foreach ($selectiveList as $key => $ar) {
                    $this->restore_archive($ar);
                }
            }
            if (count($mastIncrList)) {
                foreach ($mastIncrList as $key => $ar) {
                    $this->restore_archive($ar);
                }
            }
            if (isset($CEPRestore)) {
                $this->restore_archive($CEPRestore);
            }
            if (isset($localDirRestore)) {
                $this->restore_archive($localDirRestore);
            }
            if (count($exchange)) {
                $result = array();
                foreach ($exchange as $key => $list) {
                    $result = array_merge($result, $list);
                }
                usort($result, "compare_date");
                foreach ($result as $key => $ar) {
                    $this->restore_archive($ar);
                }
            }
            if (count($sql)) {
                $result = array();
                foreach ($sql as $key => $list) {
                    $result = array_merge($result, $list);
                }
                usort($result, "compare_date");
                foreach ($result as $key => $ar) {
                    $this->restore_archive($ar);
                }
            }
            if (count($vm)) {
                $result = array();
                foreach ($vm as $key => $list) {
                    $result = array_merge($result, $list);
                }
                usort($result, "compare_date");
                foreach ($result as $key => $ar) {
                    $this->restore_archive($ar);
                }
            }
            if (count($hyperv)) {
                $result = array();
                foreach ($hyperv as $key => $list) {
                    $result = array_merge($result, $list);
                }
                usort($result, "compare_date");
                foreach ($result as $key => $ar) {
                    $this->restore_archive($ar);
                }
            }
            if (count($xen)) {
                $result = array();
                foreach ($xen as $key => $list) {
                    $result = array_merge($result, $list);
                }
                usort($result, "compare_date");
                foreach ($result as $key => $ar) {
                    $this->restore_archive($ar);
                }
            }
        }

    }

    function restore_archive($archive) {
        global $Log;
        global $restoreClientList;
        global $deviceName;
        global $deviceID;
        $bReplace = $this->bReplace;

        if ($bReplace) {
            $clientName = $archive['client_name'];
            for ($i = 0; $i < count($restoreClientList); $i++) {
                if ($clientName == $restoreClientList[$i]) {
                    $devName = $deviceName[$i];
                    $deviceID = $this->BP->get_device_id($devName);
                }
            }
        }

        $restoreOptions = array();
        $restoreOptions['association'] = "dr";
        if ($bReplace && isset($deviceID)) {
            $restoreOptions['device_id'] = $deviceID;
        }

        $returnValue = $this->BP->restore_archived_files($archive['archive_id'], $restoreOptions);
        if ($returnValue !== false) {
            echo "JobID: $returnValue \n";
        } else {
            echo "Restore Error: " . $this->BP->getError() . "\n";
        }
    }


}
?>