<?php
// defines BP, xml, action, cookie
require 'replication-functions.php';

class Replicate {

    private $BP;
    private $dpuID;
    private $sysID;
    private $bReplace;

    function __construct($BPin){
        $this->BP = $BPin;
        // parameters passed to all API's
        $this->dpuID = isset($_GET['id']) ? (int)$_GET['id'] : $this->BP->get_local_system_id();
        $this->sysID = isset($_GET['sid']) ? (int)$_GET['sid'] : $this->BP->get_local_system_id();
    }

    function sysList(){
        $systems = $this->BP->get_system_list();

        $returner = array();
        foreach ($systems as $systemID => $systemName) {
            $source = $this->BP->get_system_info($systemID);
            if($source) {
                $sourceVersion = $source['version'];
            }
            $dpuIP = $this->BP->get_host_info($systemName);
            if($dpuIP !== false){
                $returner[] = array("Name" => $systemName, "ID" => $systemID, "IP" => $dpuIP['ip'], "version" => $sourceVersion);
            }
        }

        return $returner;
    }

    function setupTarget($targetName){
        $result = $this->BP->setup_dr_target($targetName);

        if( $result !== false) {
            return true;
        } else {
            $myErr = $this->BP->getError();
            echo "\n$myErr\n";
            return false;
        }
    }

    function clientList($sourceID){
        $grandClients = $this->BP->get_grandclient_list($sourceID);

        if ($grandClients !== false){

            return $grandClients;

        } else {
            return false;
        }
    }

    /*
     * Enqueues a metadata backup if it does not exists.
     * Returns true if successfully enqueued (or previously enqueued)
     * Returns false if not able to enqueue.
     * Returns "Completed" if the copy has completed.
     */
    function enqueueMetadata($metadata, $targetName, $dpuID){
        //check if metadata backup is already in the queue.  If true, don't try to re-add.
        $exists = $this->BP->is_backup_in_replication_queue($metadata, $targetName, $dpuID);
        if($exists !== false) {
            global $Log;
            $Log->writeVariable("exists");
            $Log->writeVariable($exists);
            if($exists == "") {
                $result = $this->BP->add_to_replication_queue($metadata, $targetName);
                return $result;
            } else if ($exists == "Completed") {
                echo "Metadata backup has already been copied to the appliance $targetName.\n";
                return $exists;
            }

        } else {
            echo "Metadata backup is already in the replication queue\n";
            return false;
        }

    }

    function clientBackups($sourceID, $targetName, $clientNames){
        global $Log;
        $result = true;
        $dpuID = $this->BP->get_local_system_id();
        $cids = buildClientListForBackups($this->BP, $clientNames, $sourceID);
        if($cids !== false){
            $backupIDs = getClientBackups($this->BP, $cids, $dpuID, 60, false, false);
            if ( $backupIDs  !== false ) {
                foreach($backupIDs as $bid) {
                    $addToQueue = $this->BP->add_to_replication_queue($bid, $targetName);
                    if (!$addToQueue){
                        $myErr = $this->BP->getError();
                        echo "$myErr\n";
                        $result = false;
                        break;
                    } else {
                        $result = $addToQueue;
                    }
                }
            } else {
                $myErr = $this->BP->getError();
                echo "$myErr\n";
                $result = false;
            }
        } else {
            $myErr = $this->BP->getError();
            echo "$myErr\n";
            $result = false;
        }
        return $result;
    }
    
    /*
     * Gets replication job info to the new appliance and waits for it to complete.
     * Sleeps 5 seconds prior to checking again. 
     * If failures are seen 5 times, returns a false result.  Otherwise, returns true when done.
     */
    function watchMetadataReplicate($metaDataBackup, $target, $dpuID) {
        $finished = false;
        $failureCount = 0;
        $result = true;
        while (!$finished) {
            $replicationJobs = $this->BP->get_replication_active_job_info(array('system_id' => $dpuID));
            if ($replicationJobs !== false) {
                if (count($replicationJobs) == 0) {
                    $finished = true;
                } else {
                    foreach ($replicationJobs as $job) {
                        if ($job['backup_id'] == $metaDataBackup && $job['target'] == $target) {
                            $status = $job['status'];
                            echo ".";
                            sleep(5);
                            if ($status !== 'active' && $status !== 'queued') {
                                $finished = true;
                                break;
                            }
                        }
                    }
                }
            } else {
                echo ".";
                $failureCount++;
                sleep(5);
                if ($failureCount > 5) {
                    $finished = true;
                    $result = false;
                }
            }        
        }
        return $result;
    }

    //if performing a DR, replication on the DR source (target of replication) needs to be enabled.
   /* function enable_replication_check(){
        $result = $this->BP->get_ini_value('Replication', 'Enabled', $this->dpuID);
        return $result;
    }*/

    
function enable_replication_check(){

    $result = $this->BP->get_ini_value('Replication', 'Enabled', $this->dpuID);
    if ($result === false) {
        return false;  // there was an error
    }
    $result = strtolower($result);
    return $result[0] == "t" || $result[0] == "y" || $result == "on" || $result == "1" ? true : false;
}

function enable_replication(){
        $result = $this->BP->set_ini_value("Replication", "Enabled", "yes", $this->dpuID);
        return $result;
    }

    /*
     * Checks to see if replication is suspended (MaxConcurrent of 0).
     */
    function isSuspended() {
        $maxConcurrent = $this->BP->get_ini_value('Replication', 'MaxConcurrent', $this->dpuID);
        if ($maxConcurrent == 0 || $maxConcurrent === false) {
            $suspended = true;
        } else {
            $suspended = false;
        }
        return $suspended;
    }
    
    /*
     * Resumes replication by setting MaxConcurrent to 2 (the default).
     */
    function resume() {
        $result = $this->BP->set_ini_value("Replication", "MaxConcurrent", 2, $this->dpuID);
        return $result;       
    }

}


?>
