<?php

class JobsActive
{
    private $BP;
    const DEFAULT_NIC = "eth0";
    const DEFAULT_TUNNEL = "tun0";
    const APPLICATION_SHARE = "application";
    const MIN_FOR_EST_DURATION = 10;

    public function __construct($BP, $sid, $Functions)
    {
        $this->BP = $BP;
        $this->now = time();
		$this->sid = $sid;
	    $this->itemCount = 0;

        $this->functions = $Functions;

        require_once('jobs.functions.php');
        $this->jobsFunctions = new JobsFunctions($this->BP);
    }


    public function get_active_jobs($which, $filter, $jobID, $sid, $systems)
    {
        $allJobs = array();

        $specialJob = explode(".", $which);

        foreach ($systems as $sid => $sysName) {
            //this handles <app_id>.ir or flr jobs types
            if (count($specialJob) > 1) {
                $allJobs = $this->getSpecialJob($specialJob, $which, $sid);
            } else {
                if ($filter === "ir") {
                    $IRJobs = $this->getIRJobs($sid);
                } else {
                    $IRJobs = array();
                }
                if ($filter === "flr") {
                    if (isset($jobID) && $jobID != null && $jobID != "") {
                        // gets details only of jobID
                        $FLRJobs = $this->getFLRJobDetails($jobID, $sid);
                    } else {
                        $FLRJobs = $this->getFLRJobs($sid);
                    }
                } else if ($filter === "rflr") {
                    /* an option used by populate_alerts.php */
                    $FLRJobs = $this->getRemoteFLRJobs();
                } else {
                    $FLRJobs = array();
                }
                if ($filter === 'target-flr') {
                    $FLRJobs = $this->getFLRJobs($sid);
                    // Get the ones for this sourceID.
                    if (isset($_GET['source_id'])) {
                        $FLRJobs = $this->getTargetFLRJobs($FLRJobs, $systems, (int)$_GET['source_id']);
                    }
                }
                if ($filter === "" || $filter === "replication" || $filter === "backup-copy") {
                    $replicationJobs = $this->getReplicationJobs($filter, $sid);
                } else {
                    $replicationJobs = array();
                }
                // get reverse replicated jobs
                if ($filter === 'target-import') {
                    $replicationJobs = $this->getReplicationJobs("replication", $sid);
                    // Get the ones for this sourceID.
                    if (isset($_GET['source_id'])) {
                        $sourceSysName = $this->functions->getSystemNameFromID((int)$_GET['source_id']);
                        $replicationJobs = $this->getTargetImportJobs($replicationJobs, $sourceSysName);
                    }
                }

                if ($filter == "" || $filter === "backup" || $filter === "restore" || $filter === "archive" || $filter === "backup-copy") {
                    $normalJobs = $this->getNormalJobs($which, $filter, $sid, $sysName);
                } else if ($filter == 'target-agent-flr') {
                    if (isset($_GET['source_id'])) {
                        $sourceSysID = (int)$_GET['source_id'];
                    } else {
                        $sourceSysID = -1;
                    }
                    //get remote agent-based flr jobs
                    $normalJobs = $this->getNormalJobs($which, "restore", $sid, $sysName, $sourceSysID);
                    // Get the ones for this sourceID.
                    if ($sourceSysID != -1) {
                        $targetName = $this->functions->getLocalSystemName();
                        $normalJobs = $this->getTargetAgentFLRJobs($normalJobs, $sourceSysID, $targetName);
                    }

                } else {
                    $normalJobs = array();
                }

                // merge all for final result.
                $allJobs = array_merge($allJobs, $IRJobs, $FLRJobs, $replicationJobs, $normalJobs);
            }
        }
        if (count($specialJob) === 0 && $filter === "" || $filter === "flr") {
            // Get any jobs on the remote (target) appliance and include them in the list.
            if ($jobID === "") {
                $allJobs = array_merge($allJobs, $this->getRemoteFLRJobs());
            } else {
                $allJobs = array_merge($allJobs, $this->getRemoteFLRJobDetails($jobID, $sid));
            }
        }
        if ($filter === "" || $filter === "replication" || $filter === "backup-copy" || $filter === "restore") {
            $allJobs = array_merge($allJobs, $this->getRemoteImportJobs(), $this->getRemoteAgentFLRJobs());
        }

        return ($allJobs);
    }

    function getNormalJobs($which, $filter, $sid, $sysName, $sourceSysID = -1)
    {
        $normalJobs = array();
        if ($sourceSysID != -1) {
            $rflr_jobs = $this->BP->get_rflr_job_list($sourceSysID, $sid);
            // making the output similar to get_job_list() (array('job ID' => 'job name'))
            foreach ($rflr_jobs as $rflr_job) {
                $jobs[$rflr_job] = "";
            }
        } else {
            $jobs = $this->BP->get_job_list($sid);
        }

        if ($jobs !== false and $jobs !== null) {
            foreach ($jobs as $id => $name) {
                if (is_numeric($which) && $which > 0 and $id != $which) {
                    continue;
                }
                if ($which) {
                    $jobInfo = $this->BP->get_job_info($id, $sid);
                    if ($jobInfo !== false) {
                        if (isset($jobInfo['client_id'])) {
                            $clientInfo = $this->BP->get_client_info($jobInfo['client_id'], $sid);
                            $clientName = $clientInfo['name'];
                        }
                        $data = $this->buildActiveJobsOutput($jobInfo, $sysName, $sid, $clientName);
                        $filtered = $this->buildReturn($data, $filter);

                        if ($filtered != NULL) {
                            $normalJobs[] = $filtered;
                        }
                    }
                } else {
                    global $Log;
                    $message = " Could not retrieve job info.";
                    $message = $this->BP->getError() . ":" . $message;
                    $Log->writeError($message, true);
                }
            }
        }

        return $normalJobs;
    }

    function buildActiveJobsOutput($jobData, $sysName, $sid, $clientName)
    {
        $data = array();

        $name = isset($jobData['name']) ? $jobData['name'] : null;
        $type = isset($jobData['type']) ? $this->jobsFunctions->getJobType($jobData['type']) : null;
        //$mode = isset($jobData['type']) ? $jobData['type'] : null;
        $mode = isset($jobData['type']) ? $this->functions->getBackupTypeDisplayName($jobData['type']) : null;
        $startDate = isset($jobData['start_time']) ? $this->functions->formatDateTime($jobData['start_time']) : null;

        $status = null;
        if (isset($jobData['status'])) {
            $status = $jobData['status'];
            if (is_int($status)) {
                // Backup Status: 0 = Successful, 1 = Warning, 2 = Failed.
                $status = ($status === 0) ? 'Successful' : (($status === 1) ? 'Warning' : 'Failed');
            }
        }

        $suspendable = isset($jobData['status']) && $jobData['status'] == 'QUEUED' ? true : false;
        $resumable = isset($jobData['status']) && $jobData['status'] == '*ON HOLD*' ? true : false;
        $instanceName = isset($jobData['instance_short_name'])  ? $jobData['instance_short_name'] : null;
        $assetName = (!strcasecmp($instanceName,'file-level')) ? $clientName : ((!strcasecmp($instanceName, 'archive')) ? "Backup Copy" : $instanceName);
        $scheduleName = isset($jobData['schedule_name']) ? $jobData['schedule_name'] : "n/a";
        $jobName = isset($scheduleName) ? $scheduleName : 'Backup of ' + $assetName;
        // Do this as the core is not returning duration.  Format duration as a string.
        $elapsedTime = isset($jobData['elapsed_time']) ? true : false;
        $duration = isset($jobData['start_time']) && $elapsedTime == false ? $this->functions->formatTimeDelta($this->now - $jobData['start_time']) : "";
        $comment = isset($jobData['comment']) ? $jobData['comment'] : null;
        $percentComplete = isset($jobData['percent_complete']) ? $jobData['percent_complete'] : null;
        if (isset($jobData['start_time']) && $percentComplete > JobsActive::MIN_FOR_EST_DURATION && $percentComplete < 100) {
            // job_start + (( 100% / % complete so far )  * (duration so far))
            $estComplete = $this->functions->formatDateTime($jobData['start_time'] + ((100.0 / $percentComplete) * ($this->now - $jobData['start_time'])));
        } else {
            $estComplete = null;
        }
        $id = isset($jobData['id']) ? $jobData['id'] : null;
        $appName = "";
        if (isset($jobData['instance_id'])) {
            $instanceID = $jobData['instance_id'];
            $instanceInfo = $this->BP->get_appinst_info($instanceID, $sid);
            if($instanceInfo != false) {
                $appName = $instanceInfo[$instanceID]['app_type'];
            }
        }

        $data = array(
            'id' => $id,
            'name' => $jobName,
            'status' => $this->mapActiveJobStatus($status),
            'comment' => $comment,
            'percent_complete' => $percentComplete,
            'start_date' => $startDate,
            'estimated_end' => $estComplete,
            'mode' => $mode,
            'type' => $type,
            'application' => $appName == "Archive" ? "Backup Copy" : $appName,
            'client_name' => $clientName,
            'instance_name' => $instanceName  == "Archive" ? "Backup Copy" : $instanceName,
            'asset_name' => $assetName,
            'duration' => $duration,
            'suspendable' => $suspendable,
            'resumable' => $resumable,
            'system_id' => $sid,
            'system_name' => $sysName
        );

        return $data;
    }

    function getRemoteAgentFLRJobs() {
        $data = array();
        $request = "GET";
        $api = "/api/jobs/active/target-agent-flr/";
        $result = $this->functions->remoteRequest("", $request, $api, "", NULL, 1);
        if (is_array($result)) {
            $jobs = $result['data'];
            foreach ($jobs as $agentFLR) {
                $agentFLR['remote'] = true;
                $data[] = $agentFLR;
            }
        }
        return $data;
    }

    function getTargetAgentFLRJobs($agentFLRJobs, $sid, $targetName) {
        $data = array();
        global $Log;
        $Log->writeVariable("getTargetAgentFLR");
        $Log->writeVariable($agentFLRJobs);
        $Log->writeVariable($sid);
        $Log->writeVariable($targetName);

        foreach ($agentFLRJobs as $item) {
            $found = false;
            if ($item['application'] == 'file-level' && ($item['asset_name'] == $targetName || $item['client_name'] == $targetName)) {
                $found = true;
            }
            if ($found) {
                $data[] = $item;
            }
        }
        return($data);
    }

    function getReplicationJobs($filter, $sid)
    {
        $replicationJobs = array();
        $allReplicationJobs = $this->BP->get_replication_active_job_info(array('system_id' => $sid));
        if($allReplicationJobs !== false and $allReplicationJobs !== null) {
            foreach ($allReplicationJobs as $job) {
                $data = $this->activeReplicationJobs($job, $sid);
                $filtered = $this->buildReturn($data, $filter);

                if ($filtered != NULL) {
                    $replicationJobs[] = $filtered;
                }
            }
        } else {
            // This will fail for any un-managed source.  Need to filter them out of $systems list, then this can be uncommented.
            //global $Log;
            //$Log->writeError('replication job info: ' . $this->BP->getError(), true);
        }
        return $replicationJobs;
    }

    function activeReplicationJobs($job, $sid)
    {
        $id = isset($job['id']) ? $job['id'] : "";
       // $clientName = isset($job['client_name']) ? $job['client_name'] : "";
        $instanceName = isset($job['instance_name']) ? $job['instance_name'] : "";
        $systemName = isset($job['system_name']) ? $job['system_name'] : "";
        $name = isset($job['schedule_name']) ? $job['schedule_name'] : "Replication";
        $type = isset($job['type']) ? $job['type'] : "";
        $mode = isset($job['mode']) ? $this->functions->getBackupTypeDisplayName($job['mode']) : "";
        $sourceSID = isset($job['sid']) ? $job['sid'] : "";
        $phaseName = isset($job['name']) ? $job['name'] : "";
        $target = isset($job['target']) ? $job['target'] : "";
        $backupID = isset($job['backup_id']) ? $job['backup_id'] : "";
        $size = isset($job['size']) ? $job['size'] : "";
        $dataWritten = isset($job['data_written']) ? $job['data_written'] : "";
        //$startTime = isset($job['start_time']) ? $job['start_time'] : "";
        $startDate = isset($job['start_time']) && $job['start_time'] > 0 ? $this->functions->formatDateTime($job['start_time']) : null;
        $duration = isset($job['duration']) ? $this->functions->formatTimeDelta($job['duration']) : null;
        $percentComplete = isset($job['percent_complete']) ? $job['percent_complete'] : "";
        if (isset($job['start_time']) && $percentComplete != "" && $duration !== null &&
            $percentComplete > JobsActive::MIN_FOR_EST_DURATION && $percentComplete < 100) {
            // job_start + (( 100% / % complete so far )  * (duration so far))
            $estComplete = $this->functions->formatDateTime($job['start_time'] + ((100.0 / $percentComplete) * $job['duration']));
        } else {
            $estComplete = null;
        }
        $phaseStartDate = isset($job['phase_start_date']) ? $job['phase_start_date'] : "";
        $phaseCompletion = isset($job['phase_completion']) ? $job['phase_completion'] : "";
        $phaseElapsedTime = isset($job['phase_elapsed_time']) ? $job['phase_elapsed_time'] : "";
        $status = isset($job['status']) ? $job['status'] : "";
        $suspendable = isset($job['suspendable']) ? $job['suspendable'] : "";
        $cancellable = isset($job['cancellable']) ? $job['cancellable'] : "";
        $instanceID = isset($job['instance_id']) ? $job['instance_id'] : null;
       /// $appInst = $this->BP->get_appinst_info($instanceID, $sid);

        if ($mode !== "system metadata") {
           /* if ($appInst !== false and $instanceID !== null) {
                foreach ($appInst as $appInfo) {
                    $app_type = isset($appInfo['app_type']) ? $appInfo['app_type'] : "n/a";
                    $clientName = $app_type == "Hyper-V" ? $appInfo['client_name'] : $appInfo['primary_name'];
                    $instanceName = $app_type == "Hyper-V" ? $appInfo['primary_name'] : $appInfo['secondary_name'];
                }
            }*/

            $instanceNames = $this->functions->getInstanceNames($instanceID, $sid);
            if ($instanceNames !==false){
                $appType = $instanceNames['app_type'];
                $clientName = $instanceNames['client_name'];
                $instanceName = $instanceNames['asset_name'];
            }

            $data = array(
                'id' => $id . ".rep",
                'client_name' => $clientName,
                'application' => $appType,
                'instance_name' => $instanceName,
                'system_name' => $systemName,
                'type' => $type,
                'mode' => $mode,
                'sid' => $sourceSID,
                'name' => $name,
                'asset_name' => $instanceName,
                'target' => $target,
                'backup_id' => $backupID,
                'size' => $size,
                'data_written' => $dataWritten,
                'start_date' => $startDate,
                'duration' => $duration,
                'percent_complete' => $percentComplete,
                'estimated_end' => $estComplete,
                'phase_start_date' => $phaseStartDate,
                'phase_completion' => $phaseCompletion,
                'phase_elapsed_time' => $phaseElapsedTime,
                'status' => $status,
                'suspendable' => $suspendable,
                'cancellable' => $cancellable,
                'comment' => $phaseName
            );
            return $data;
        }
    }

    function getRemoteImportJobs() {
        $data = array();
        $request = "GET";
        $api = "/api/jobs/active/target-import/";
        $result = $this->functions->remoteRequest("", $request, $api, "", NULL, 1);
        if (is_array($result)) {
            $jobs = $result['data'];
            foreach ($jobs as $import) {
                $import['remote'] = true;
                $data[] = $import;
            }
        }
        return $data;
    }

    function getTargetImportJobs($importJobs, $sourceSysName) {
        $data = array();
        global $Log;
        $Log->writeVariable("getTargetImport");
        $Log->writeVariable($importJobs);
        $Log->writeVariable($sourceSysName);

        foreach ($importJobs as $item) {
            $found = false;
            $targetName = $item['target'];
            if ($targetName !== "" && $targetName === $sourceSysName) {
                $found = true;
            }
            if ($found) {
                $data[] = $item;
            }
        }
        return($data);
    }

    function getSpecialJob($specialJob, $which, $sid){
        if($specialJob[1] == "ir") {
            $tempJobs = $this->getIRJobs($sid);
        } elseif($specialJob[1] == "flr") {
            $tempJobs = $this->getFLRJobs($sid);
        } elseif ($specialJob[1] == "target") {
            $tempJobs = $this->getJobOnTarget($specialJob, $sid);
        }
        foreach($tempJobs as $job){
            if($job['id'] == $which){
                $allJobs = $job;
            }
        }

        return $allJobs;
    }
    function buildIROutput($IRJobs, $sid, $type){

        $id = isset($IRJobs['id']) ? $IRJobs['id'] : null;
        $status = isset($IRJobs['status']) ? $IRJobs['status'] : null;
        $detail = isset($IRJobs['detail']) ? $IRJobs['detail'] : null;
        $percentage = ($status == 'error' || $status == 'dead') ? 0 : ($status == 'available' ? 100 : isset($IRJobs['percentage']) ? $IRJobs['percentage'] : null);
        $time = isset($IRJobs['time']) ? $this->functions->formatDateTime($IRJobs['time']) : null;
        $duration = isset($IRJobs['time']) ? $this->functions->formatTimeDelta($this->now - $IRJobs['time']) : null;
        $audit = isset($IRJobs['audit']) ? $IRJobs['audit'] == 1 ? "true" : "false": "n/a";

        $instanceName = isset($IRJobs['vm_name']) ? $IRJobs['vm_name'] : null;
        $client = isset($IRJobs['vm_server']) ? $IRJobs['vm_server'] : null;
        $appInst = $this->BP->get_appinst_info($id, $sid);
        if ($appInst !== false and $id !== null){
            foreach($appInst as $appInfo) {
                $type = isset($appInfo['app_type']) ? $appInfo['app_type'] : "n/a";
            }
        }
        $data = array(
            'id' => $id . ".ir",
            'name' => "IR of " . $instanceName,
            'status' => $status,
            'comment' => $detail,
            'percent_complete' => $percentage,
            'start_date' => $time,
            'mode' => "Instant Recovery",
            'type' => "restore",
            'application' => $type,
            'client_name' => $client,
            'instance_name' => $instanceName,
            'asset_name' => $instanceName,
            'duration' => $duration,
            'audit' => $audit,
            'unique' => $audit,
            'suspendable' => false,
            'resumable' => false,
            'system_id' => $sid,
            'system_name' => $this->functions->getSystemNameFromID($sid)
        );

        return $data;
    }

    function getIRJobs($sid){
        $allJobs = array();
        $vmIRStatus = $this->BP->get_vm_ir_status($sid);
        $hvIRStatus = $this->BP->get_hyperv_ir_status($sid);
        if ($vmIRStatus !== false) {
            foreach ($vmIRStatus as $IR) {
                $type = "VMware";
                $IRJobs = $this->buildIROutput($IR, $sid, $type);
                if(!empty($IRJobs)) {
                    $allJobs[] = $IRJobs;
                }
            }
        }
        if ($hvIRStatus !== false) {
            $type = "Hyper-V";
            foreach ($hvIRStatus as $IR) {
                $IRJobs = $this->buildIROutput($IR, $sid, $type);
                if(!empty($IRJobs)) {
                    $allJobs[] = $IRJobs;
                }
            }
        }
        return $allJobs;
    }

    function getFLRJobs($sid){
        $allJobs = array();

        // Get the appliance IP to show as the base of the mounted shares.
        $network = $this->BP->get_network_info(JobsActive::DEFAULT_NIC, $sid);
        $appliance_nic = ($network !== false) ? $network['ip'] : '{this_appliance_ip}';

        // Get tunnel IP to get the openvpn share details
        $tunnel = $this->BP->get_network_info(JobsActive::DEFAULT_TUNNEL, $sid);
        $tunnel_nic = ($tunnel !== false) ? $tunnel['ip'] : null;

        $diskImageStatus = $this->BP->get_disk_image_status($sid);
        if ($diskImageStatus !== false) {
            foreach ($diskImageStatus as $FLR) {
                $shareType = "image";
                $FLRJobs = $this->buildFLROutput($FLR, $shareType, $appliance_nic, $tunnel_nic, $sid);
                $allJobs[] = $FLRJobs;
            }
        }

        $mountStatus = $this->BP->backup_mount_status($sid);
        if ($mountStatus !== false && isset($mountStatus['status']) && $mountStatus['status'] !== "none") {
            $shareType = "mount";
            $FLRJobs = $this->buildFLROutput($mountStatus, $shareType, $appliance_nic, $tunnel_nic, $sid);
            $allJobs[] = $FLRJobs;
        }

        $applicationShareStatus = $this->BP->get_application_share_status($sid);
        if ($applicationShareStatus !== false){
            foreach($applicationShareStatus as $appShare) {
                $shareType = JobsActive::APPLICATION_SHARE;
                $FLRJobs = $this->buildFLROutput($appShare, $shareType, $appliance_nic, $tunnel_nic, $sid);
                $allJobs[] = $FLRJobs;
            }
        }

        return $allJobs;
    }

    function getFLRJobDetails ($jobID, $sid)
    {
        $jobDetails = array();
        $allFLRJobs = $this->getFLRJobs($sid);
        foreach ($allFLRJobs as $flrJob) {
            // filter out the one required job from the list of all jobs
            if ($jobID === $flrJob['id']) {
                $jobDetails = $flrJob;
            }
        }
        return $jobDetails;
    }

    function getRemoteFLRJobs() {
        $data = array();
        $request = "GET";
        $api = "/api/jobs/active/target-flr/";
        $result = $this->functions->remoteRequest("", $request, $api, "", NULL, 1);
        if (is_array($result)) {
            $jobs = $result['data'];
            foreach ($jobs as $flr) {
                $flr['remote'] = true;
                $flr['asset_name'] = $flr['asset_name'] . ' (on the target)';
                $data[] = $flr;
            }
        }
        return $data;
    }

    function getRemoteFLRJobDetails($jobID)
    {
        $flrDetails = array();
        $allFLRJobs = $this->getRemoteFLRJobs();
        foreach ($allFLRJobs as $flrJob) {
            if ($jobID === $flrJob['id']) {
                $jobDetails = $flrJob;
                $flrDetails = $this->getFLRProxyMount($jobDetails);
            }
        }
        return $flrDetails;
    }

    function getFLRProxyMount($jobDetails)
    {
        $flrDetails = $jobDetails;
        $specialJob = explode(".", $jobDetails['id']);
        $id = $specialJob[0];
        if (isset($jobDetails['restore_path'])) {
            if (isset($jobDetails['share_details']) || isset($jobDetails['tunnel_share_details'])) {
                // check if proxy mount exists; if it does, update the FLR job details; otherwise create a proxy mount
                $mount_path = $this->BP->get_proxy_mount_path($id);

                if ($mount_path === false) {
                    $path = "";
                    // converting the windows style path to linux style
                    if (isset($jobDetails['tunnel_share_details'])) {
                        $path = $this->convertToLinuxStyle($jobDetails['tunnel_share_details']);
                    } else {
                        $path = $this->convertToLinuxStyle($jobDetails['share_details']);
                    }
                    //creating the proxy mount
                    $create_proxy = $this->BP->create_proxy_mount($id, $path);
                    if ($create_proxy !== false) {
                        $mount_path = $this->BP->get_proxy_mount_path($id);
                        $flrDetails['share_details'] = $mount_path;
                        $flrDetails['available_targets'] = "smb";
                        $flrDetails['iscsi_details'] = null;
                        $flrDetails['comment'] = null;
                    } else {
                        $result['error'] = 500;
                        $result['message'] = "Unable to create proxy mount.";
                        return $result;
                    }
                } else {
                    $flrDetails['share_details'] = $mount_path;
                    $flrDetails['available_targets'] = "smb";
                    $flrDetails['iscsi_details'] = null;
                    $flrDetails['comment'] = null;
                }
            }
            if (!strstr($jobDetails['available_targets'], 'smb')) {
                $flrDetails['iscsi_details'] = null;
                $flrDetails['comment'] = "Target exported mounts are not available. Use 'Browse/Download'.";
            }
        } else {
            $flrDetails['iscsi_details'] = null;
        }
        return $flrDetails;
    }

    function buildFLROutput($FLRjobs, $shareType, $appliance_nic, $tunnel_nic, $sid) {

        if ($shareType === JobsActive::APPLICATION_SHARE){
            $appInstID = isset($FLRjobs['appinst_id']) ? $FLRjobs['appinst_id'] : null;

            if(isset($FLRjobs['type'])){
                $appShareType = $FLRjobs['type'];
                $appShareStatus = $appShareType == 'restore' ? 'available' : 'In use for backup';
            }
        }
        $id = isset($FLRjobs['id']) ? $FLRjobs['id'] : (($shareType === JobsActive::APPLICATION_SHARE) ? $appInstID : null);

        $status = isset($FLRjobs['status']) ? $FLRjobs['status'] : null;

        $time = isset($FLRjobs['available_since']) ? $this->functions->formatDateTime($FLRjobs['available_since']) :
                (isset($FLRjobs['creation_date']) ? $this->functions->formatDateTime($FLRjobs['creation_date']) : null);

        $duration = isset($FLRjobs['available_since']) ? $this->functions->formatTimeDelta($this->now - $FLRjobs['available_since']) :
                    (isset($FLRjobs['creation_date']) ? $this->functions->formatTimeDelta($this->now - $FLRjobs['creation_date']) : null);

        $availableTargets = isset($FLRjobs['available_targets']) ? $FLRjobs['available_targets'] : null;

        $restorePath = isset($FLRjobs['restore_path']) ? $FLRjobs['restore_path'] : null;

        $adminUrl = isset($FLRjobs['appinfo']) ? $FLRjobs['appinfo'] : null;

        $message = isset($FLRjobs['message']) ? $FLRjobs['message'] : null;

        $percentage = null;
        if (strcasecmp($status, 'available') == 0) {
            $percentage = 100;
        } else if (strcasecmp($status, 'error') == 0) {
            $percentage = 0;
        }
        $shareDetails = null;
        $iscsiDetails = null;
        if (strstr($availableTargets, 'smb')) {
            $shareDetails = "\\\\" . $appliance_nic . "\\" . 'flr' . $id;
            if ($tunnel_nic != null) {
                $tunnelShareDetails = "\\\\" . $tunnel_nic . "\\" . 'flr' . $id;
            }
        }
        if (strstr($availableTargets, 'iscsi')) {
            $iscsiDetails =  $appliance_nic . "/iscsi_flr";
            if ($tunnel_nic != null) {
                $tunnelIscsiDetails = $tunnel_nic . "/iscsi_flr";
            }
        }

        $appArray = $this->BP->get_application_clients($id, $sid);
        $appName = $appArray !== false ? $appArray['app_name'] : null;

        $appInst = $this->BP->get_appinst_info($id, $sid);
        if ($appInst !== false and $id !== null){
            foreach($appInst as $appInfo) {
                $app_type = isset($appInfo['app_id']) ? $this->functions->getApplicationTypeFromApplictionID($appInfo['app_id']) : "";
                $type = isset($appInfo['app_type']) ? $appInfo['app_type'] : "n/a";
                $client = $type == "Hyper-V" ? $appInfo['client_name'] : $appInfo['primary_name'];
                $instanceName = $type == "Hyper-V" ? $appInfo['primary_name'] : $appInfo['secondary_name'];
            }
        }

        if ($shareType === JobsActive::APPLICATION_SHARE){
            $imageName = isset($FLRjobs['name']) ? $FLRjobs['name'] : "";
            $instanceNames = $this->functions->getInstanceNames($id, $sid);
            if ($instanceNames !==false){
                $client = $instanceNames['client_name'];
                $instanceName = $instanceNames['asset_name'];
            }

            $shareDetails = "\\\\" . $appliance_nic . "\\"  . $imageName;
            if ($tunnel_nic != null) {
                $tunnelShareDetails = "\\\\" . $tunnel_nic . "\\" . $imageName;
            }
            $availableTargets = "smb";
            if ($adminUrl != null) {
                $adminUrlArray = array('admin_url' => $adminUrl);
            }
        }

        //handle backup_mount_status, always Exchange
        if ($shareType === "mount"){
            $instanceName = "Exchange";
            $shareDetails = "\\\\" . $appliance_nic . "\\exchange_restore";
            if ($tunnel_nic != null) {
                $tunnelShareDetails = "\\\\" . $tunnel_nic . "\\exchange_restore";
            }
            $availableTargets = "smb";
            $app_type = $instanceName;
        }

        $data = array(
            'id' => $id . ".flr",
            'name' => "FLR of " . $instanceName,
            'status' => $shareType == JobsActive::APPLICATION_SHARE ? $appShareStatus : $status,
            'comment' => $message,
            'percent_complete' => $percentage,
            'start_date' => $time,
            'mode' => "File-Level Recovery",
            'type' => "Restore",
            'application' => $appName,
            'client_name' => $client,
            'instance_name' => $instanceName,
            'asset_name' => $instanceName,
            'duration' => $duration,
            'available_targets' => $availableTargets,
            'restore_path' => $restorePath,
            'share_details' => $shareDetails,
            'iscsi_details' => $iscsiDetails,
            'unique' => $availableTargets,
            'suspendable' => false,
            'resumable' => false,
            'system_id' => $sid,
            'system_name' => $this->functions->getSystemNameFromID($sid),
            'app_type' => $app_type
        );

        if ($adminUrl != null) {
            $data = array_merge($data, $adminUrlArray);
        }

        if (isset($tunnelShareDetails) && $tunnelShareDetails != null) {
            $tunnelShareDetailsArray = array('tunnel_share_details' => $tunnelShareDetails);
            $data = array_merge($data, $tunnelShareDetailsArray);
        }

        if (isset($tunnelIscsiDetails) && $tunnelIscsiDetails != null) {
            $tunnelIscsiDetailsArray = array('tunnel_iscsi_details' => $tunnelIscsiDetails);
            $data = array_merge($data, $tunnelIscsiDetailsArray);
        }

        return $data;
    }

    function getTargetFLRJobs($FLRJobs, $systems, $source_id) {
        $data = array();
        global $Log;
        $Log->writeVariable("getTargetFLR");
        $Log->writeVariable($FLRJobs);
        $Log->writeVariable($source_id);

        /* obtain the timeout for remote FLRs sessions, used for generating alerts on the source */
        $current_time     = time();
        $rflr_timeout_hrs = (int) $this->BP->get_ini_value("SelfService", "MaxRunningTimePerFLRHrs");
        if ($rflr_timeout_hrs < 0) $rflr_timeout_hrs = 72; # don't allow negative time, 0 good for testing!
        $rflr_timeout_secs = $rflr_timeout_hrs * 3600;
        $Log->writeVariableDBG("rflr_warn_timeout_secs:".$rflr_timeout_secs);

        /* obtain the grace timeout for remote FLRs sessions, used for purging FLRs on the target */
        $rflr_graceperiod_hrs = (int) $this->BP->get_ini_value("SelfService", "GracePeriodPerFLRHrs");
        if ($rflr_graceperiod_hrs < 0) $rflr_graceperiod_hrs = 24; # don't allow negative time, 0 good for testing!
        $rflr_graceperiod_secs = $rflr_graceperiod_hrs * 3600;
        $rflr_purge_timeout_secs = $rflr_graceperiod_secs + $rflr_timeout_secs;
        $Log->writeVariableDBG("rflr_purge_timeout_secs:".$rflr_purge_timeout_secs);

        foreach ($FLRJobs as $item) {
            $found = false;
            $id = (int)$item['id'];
            $appInfo = $this->BP->get_appinst_info($id);
            if ($appInfo !== false) {
                $client_id = $appInfo[$id]['client_id'];
                $clients = $this->BP->get_grandclient_list($source_id);
                foreach ($clients as $cid => $clientName) {
                    if ($cid === $client_id) {
                        $found = true;
                        break;
                    }
                }
            }
            if ($found) {
                /* calculate the expiration time, to be used for alerts and purges within populate_alerts.php */
                $startDateStr    = $item['start_date'];
                $available_since = strtotime($startDateStr);
                $time_online     = $current_time - $available_since;
                if ($time_online >= $rflr_purge_timeout_secs) {
                    $item['expired_purge'] = true;
                }
                else {
                    $item['expired_purge'] = false;
                    if ($time_online >= $rflr_timeout_secs) {
                        $item['expired_warn'] = true;
                    }
                    else {
                        $item['expired_warn'] = false;
                    }
                }
                $data[] = $item;
            }
        }

        return($data);
    }

    function getJobOnTarget($specialJob, $sid) {
        $data = array();
        $jobID = $specialJob[0];
        $request = "GET";
        $api = "/api/jobs/" . $jobID . "/";
        $parameters = "";
        $result = $this->functions->remoteRequest("", $request, $api, $parameters, NULL, 1);
        if (is_array($result) and isset($result['data'])) {
            $data = $result['data'];
            $data['remote'] = true;
        }

        return $data;
    }

    function buildReturn($data, $filter){
        $skip = false;

        if ($filter != "") {
            switch ($filter) {

                // get them all from jobs list.
                case 'active':
                    break;
                // see if type matches user selection.  If not, skip it.
                default:
                    if ($filter == 'backup-copy' && ($data['type'] == 'archive' || $data['type'] == 'replication')) {
                        // include both archive and replication jobs in backup-copy filter.
                    } else if ($data['type'] != $filter) {
                        $skip = true;
                    }
                    break;
            }
        }
        if ($skip) {
            $data = NULL;
        }

        return $data;
    }

    function processActiveJobs($which, $filter, $sid, $sysName)
    {
        $allJobs = array();
        $jobs = $this->BP->get_job_list($sid);

        foreach ($jobs as $id => $name) {
            if (is_numeric($which) && $which > 0 and $id != $which) {
                continue;
            }
            if ($which) {
                $jobInfo = $this->BP->get_job_info($id, $sid);
                if (isset($jobInfo['client_id'])) {
                    $clientInfo = $this->BP->get_client_info($jobInfo['client_id'], $sid);
                    $clientName = $clientInfo['name'];
                }

                $data = $this->buildOutput($jobInfo, $sysName, $sid, $clientName, $which);
                $filtered = $this->buildReturn($data, $filter);

                if ($filtered != NULL) {
                    $allJobs['data'][] = $filtered;
                }
            }
        }
        return $allJobs;
    }

    function mapActiveJobStatus($status) {
        $status_map = array('*ON HOLD*' => 'Paused',
                            'QUEUED' => 'Queued',
                            'CONNECTING' => 'Connecting',
                            ' *ACTIVE*' => 'Active',
                            'ACTIVE' => 'Active',
                            'DB. UPDATING' => 'DB Update',
                            'CANCELLED' => 'Cancelled',
                            'SUCCESS*' => 'Warning',
                            'SUCCESSFUL' => 'Successful',
                            ' FAILED' => 'Error',
                            'COMPLETE' => 'Complete',
                            'PROCESS TERMINATED' => 'Terminated',
                            'UNKNOWN' => 'Unknown',
                            'INCOMPLETE' => 'Incomplete',
                            'UNKNOWN/Cancelled' => 'Unknown/Cancelled',
                            'POSITIONING' => 'Positioning',
                            'CHANGE VOL.' => 'Volume Changing');

        return ($status_map[$status]);

    }

    function isJobCompleted($status) {
        $jobCompleted = false;
        if ($status == 'CANCELLED' || $status == 'SUCCESS*' || $status == 'SUCCESSFUL' || $status == ' FAILED' || $status == 'COMPLETE' ||
            $status == 'PROCESS TERMINATED' || $status == 'INCOMPLETE' || $status == 'UNKNOWN/Cancelled') {
            $jobCompleted = true;
        }

        return $jobCompleted;
    }

    function convertToLinuxStyle($details)
    {
        $linux_style_details = str_replace("\\", "/", $details);
        return $linux_style_details;
    }

} //end JobsActive class


