<?php

$dir = '/api/includes/reports';

class ReplicationCapacityReport
{
    private $BP;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }

    public function get_replication_capacity_report($which, $sid, $data)
    {
        $report = array();
        if ($which === "capacity") {
            $report = $this->get_replication_target_capacity_report($sid, $data);
            if ($report['count'] > 0) {
                // do nothing
            } else {
                $report = $this->get_replication_source_capacity_report($sid, $data);
            }
        } else if ($which === "target-capacity") {
            $report = $this->get_replication_target_capacity_report($sid, $data);
            //filter out report data based on source ID
            if (isset($_GET['source_id'])) {
                $sourceSysName = $this->functions->getSystemNameFromID((int)$_GET['source_id']);
                $report = $this->filter_replication_source_capacity_report($report, $sourceSysName);
            }
        }
        return $report;
    }

    // report that's returned on the target system
    private function get_replication_target_capacity_report($sid, $data)
    {
        $targetSelected = false;
        $localSystemID = $this->BP->get_local_system_id();
        if ($sid === $localSystemID) {
            // If the target is selected, get all replicating assets, summarizing size per appliance.
            $sid = false;
            $targetSelected = true;
        }
        $systems = $this->functions->selectReplicatingSystems($sid);

        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();

        foreach ($systems as $system_id => $system_name)
        {
            $system_size = 0;
            // Get retention for all replicated assets on this target, per source/system_id
            $retentionInfoList = $this->BP->get_replicated_lastfull_info(array('system_id' => $system_id));
            if ($retentionInfoList !== false)
            {
                foreach ($retentionInfoList as $retentionInfo)
                {
                    if (isset($retentionInfo['app_name']) && isset($retentionInfo['instance_id'])) {
                        $appName = $retentionInfo['app_name'];
                        if ($appName !== Constants::APPLICATION_TYPE_NAME_SYSTEM_METADATA) {
                            // Not including Appliance Metadata in the capacity report.
                            $instanceID = $retentionInfo['instance_id'];
                            $instanceInfo = $this->functions->getInstanceNames($instanceID, $localSystemID);
                            if ($instanceInfo !== false) {
                                $instanceName = $instanceInfo['asset_name'];
                                $clientName = $instanceInfo['client_name'];
                                if (!$targetSelected) {
                                    $report_data[] = $this->buildReplicationCapacityPerInstance($appName,
                                        $clientName,
                                        $retentionInfo['mb_size'],
                                        $system_name,
                                        $instanceName);
                                    $report_count += 1;                                    
                                } else {
                                    $system_size += $retentionInfo['mb_size'];
                                }
                            }
                        }
                    }
                }
                if ($targetSelected) {
                    // Output one summary record per replicating source.
                    $report_data[] = $this->buildReplicationCapacityPerInstance('',
                        'All',
                        $system_size,
                        $system_name,
                        'All');
                    $report_count += 1;
                }
            } else {
                // Output a row with any system that fails the retention info call.
                $message = $this->BP->getError();
                $report_data[] = $this->buildReplicationCapacityPerInstance($message, 'unavailable', '', $system_name);
                $report_count += 1;
            }
        }
        $report_total = $report_count;  //Temporary

        $report['count'] = $report_count;
        $report['total'] = $report_total;
        $report['start'] = $report_start;
        $report['data'] = $report_data;
        return $report;

    }

    // report run on the source that gets the source's info from the target
    private function get_replication_source_capacity_report($sid, $data)
    {
        $report = array();
        // phase 1 - this report will be supported only in case of source-target and not in cross-vault cases
        $systems = $this->functions->selectReplicatingSystems($sid);
        if (count($systems) == 0) {
            $request = "GET";
            $api = "/api/reports/replication/target-capacity/";
            $result = $this->functions->remoteRequest("", $request, $api, "");
            if (is_array($result)) {
                $report = $result;
                // include 'remote=true' in the report data to differentiate that these records are from the target
                $report['data'] = array();
                foreach ($result['data'] as $report_data) {
                    $report_data['remote'] = true;
                    $report['data'][] = $report_data;
                }
            }
        } else {
            $report['data'] = array();
            $report['count'] = $report['total'] = $report['start'] = 0;
        }
        return $report;
    }

    // based on the source name, this function filters the results got back from the target to include just those pertaining to the source where the report is being run
    private function filter_replication_source_capacity_report($report, $sourceSysName)
    {
        $data = array();
        $report_count = 0;
        global $Log;
        $Log->writeVariable("filter_replication_source_capacity_report");
        $Log->writeVariable($report);
        $Log->writeVariable($sourceSysName);

        foreach ($report['data'] as $item) {
            $found = false;
            $sysName = $item['system_name'];            // name of the source system as displayed on the target
            if ($sysName !== "" && $sysName === $sourceSysName) {
                $found = true;
            }
            if ($found) {
                $data['data'][] = $item;
                $report_count += 1;
            }
        }
        // if any data pertaining to the source is found, add the other details to the result
        if ($report_count > 0) {
            $data['count'] = $report_count;
            $data['start'] = $report['start'];
            $data['total'] = $report_count;
        }
        $Log->writeVariable($data);
        return $data;
    }

    private function buildReplicationCapacityPerInstance( $appName, $clientName, $size, $systemName, $instanceName = '', $databaseName = '')
    {
        $replication_capacity_info = array();
        $replication_capacity_info['app_name'] = $appName;
        $replication_capacity_info['client_name'] = $clientName;
        $replication_capacity_info['instance_name'] = $instanceName;
        $replication_capacity_info['database_name'] = $databaseName;
        $replication_capacity_info['size'] = $size;
        $replication_capacity_info['system_name'] = $systemName;
        //$replication_capacity_info['retention_policy'] = 'CoreWorkNeeded'; //Not available for 1st Out
        //$replication_capacity_info['retention_id'] = 0; //Not available for 1st Out

        return $replication_capacity_info;
    }

}