<?php

$dir = '/api/includes/reports';

class SystemCapacityReport
{
    private $BP;
    const UNKNOWN_ELEMENT = "Unknown";

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }

    public function get_system_capacity_report($systems)
    {
        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();

        foreach ($systems as $systemID => $systemName)
        {
            $retentionInfo = $this->BP->get_retention_info($systemID);
            if ($retentionInfo !== false and count($retentionInfo) > 0 )
            {
                $temp_system_report = array();
                $temp_system_report['system_id'] = $systemID;
                $temp_system_report['system_name'] = $systemName;
                $temp_system_report['used'] = $retentionInfo['used'];
                $temp_system_report['available'] = $retentionInfo['available'];
                $temp_system_report['message'] = ( array_key_exists('message', $retentionInfo) ? $retentionInfo['message'] : "" );
                $vfSpaceInfo = $this->BP->get_virtual_failover($systemID);
                $temp_system_report['ir_used'] = (array_key_exists('allowed', $vfSpaceInfo) ? $vfSpaceInfo['allowed'] : 0);
                $temp_system_report['clients'] = array();
                $temp_system_report['vms'] = array();
                $temp_system_report[Constants::APPLICATION_TYPE_NAME_EXCHANGE] = array();
                $temp_system_report[Constants::APPLICATION_TYPE_NAME_SQL_SERVER] = array();
                $temp_system_report[Constants::APPLICATION_TYPE_NAME_SHAREPOINT] = array();
                $temp_system_report[Constants::APPLICATION_TYPE_NAME_ORACLE] = array();
                $temp_system_report[Constants::APPLICATION_TYPE_NAME_NDMP_DEVICE] = array();
                $temp_system_report[Constants::APPLICATION_TYPE_NAME_UCS_SERVICE_PROFILE] = array();
                $temp_system_report[Constants::BACKUP_DISPLAY_TYPE_BAREMETAL] = array();

                if ( array_key_exists('clients', $retentionInfo) and count($retentionInfo['clients']) > 0 )
                {
                    foreach ( $retentionInfo['clients'] as $client )
                    {
                        if ( $client['name'] !== Constants::CLIENT_NAME_VCENTER_RRC )
                        {
                            $isSystemClient = false;
                            if ( $client['name'] === $systemName )
                            {
                                $tempClientID = $this->BP->get_client_id( $client['name'], $systemID );
                                $isSystemClient = ( $tempClientID === 1 );
                            }
                            if ( $isSystemClient === false )
                            {
                                $temp_client_array = array();
                                $temp_client_array['name'] = $temp_client_array['host_name'] = $client['name'];
                                $temp_client_array['last_full'] = $client['last_master'];
                                $temp_client_array['num_fulls'] = $client['num_masters'];

                                if (array_key_exists('last_baremetal', $client))
                                {
                                    $last_baremetal_size = $client['last_baremetal'];
                                    if ($last_baremetal_size > 0) {
                                        $temp_baremetal_array = array();
                                        $temp_baremetal_array['name'] = $temp_baremetal_array['host_name'] = $client['name'];
                                        $temp_baremetal_array['last_full'] = $last_baremetal_size;
                                        $temp_system_report[Constants::BACKUP_DISPLAY_TYPE_BAREMETAL][] = $temp_baremetal_array;
                                        $report_count += 1;
                                    }
                                }

                                /*
                                 * Exchange is currently handled differently than other apps, using the last_exchange index.
                                 * To keep the organizational structure like other apps, build an exchange array of one element that looks like the others.
                                 * Maybe we can get the info on each database at some later date.
                                 */
                                if (array_key_exists('last_exchange', $client))
                                {
                                    $exchange_full_size = $client['last_exchange'];
                                    if ($exchange_full_size > 0) {
                                        // build single array element of info.
                                        $exchangeRetentionInfo = array( 'database_name' => $client['name'],
                                            'last_full' => $exchange_full_size);
                                        $exchangeInfo = array($exchangeRetentionInfo);
                                        foreach ($exchangeInfo as $exchange)
                                        {
                                            $temp_exchange_array = array();
                                            $temp_exchange_array['name'] = array_key_exists('database_name', $exchange) ?  $exchange['database_name'] : SystemCapacityReport::UNKNOWN_ELEMENT;
                                            $temp_exchange_array['last_full'] = $exchange['last_full'];
                                            $temp_exchange_array['host_name'] = $client['name'];

                                            $temp_system_report[Constants::APPLICATION_TYPE_NAME_EXCHANGE][] = $temp_exchange_array;
                                            $report_count += 1;
                                        }
                                    }
                                }
                                if (array_key_exists('sql', $client))
                                {
                                    $sqlInfo = $client['sql'];
                                    foreach ($sqlInfo as $sql) {
                                        $temp_sql_array = array();
                                        $temp_sql_array['instance_name'] = array_key_exists('instance_name', $sql) ?  $sql['instance_name'] : SystemCapacityReport::UNKNOWN_ELEMENT;
                                        $temp_sql_array['name'] = array_key_exists('database_name', $sql) ? $sql['database_name'] : SystemCapacityReport::UNKNOWN_ELEMENT;
                                        $temp_sql_array['last_full'] = $sql['last_full'];
                                        $temp_sql_array['host_name'] = $client['name'];
                                        
                                        $temp_system_report[Constants::APPLICATION_TYPE_NAME_SQL_SERVER][] = $temp_sql_array;
                                        $report_count += 1;
                                    }
                                }
                                if (array_key_exists('sharepoint', $client))
                                {
                                    $sharepointInfo = $client['sharepoint'];
                                    foreach ($sharepointInfo as $sharepoint)
                                    {
                                        $temp_sharepoint_array = array();
                                        $temp_sharepoint_array['instance_name'] = array_key_exists('instance_name', $sharepoint) ? $sharepoint['instance_name'] : SystemCapacityReport::UNKNOWN_ELEMENT;
                                        $temp_sharepoint_array['name'] = $temp_sharepoint_array['instance_name'];
                                        $temp_sharepoint_array['last_full'] = $sharepoint['last_full'];
                                        $temp_sharepoint_array['host_name'] = $client['name'];
                                        
                                        $temp_system_report[Constants::APPLICATION_TYPE_NAME_SHAREPOINT][] = $temp_sharepoint_array;
                                        $report_count += 1;
                                    }
                                }
                                if (array_key_exists('oracle', $client))
                                {
                                    $oracleInfo = $client['oracle'];
                                    foreach ($oracleInfo as $oracle)
                                    {
                                        $temp_oracle_array = array();
                                        $temp_oracle_array['instance_name'] = array_key_exists('instance_name', $oracle) ? $oracle['instance_name'] : SystemCapacityReport::UNKNOWN_ELEMENT;
                                        $temp_oracle_array['name'] = $temp_oracle_array['instance_name'];
                                        $temp_oracle_array['last_full'] = $oracle['last_full'];
                                        $temp_oracle_array['host_name'] = $client['name'];
                                        
                                        $temp_system_report[Constants::APPLICATION_TYPE_NAME_ORACLE][] = $temp_oracle_array;
                                        $report_count += 1;
                                    }
                                }
                                if (array_key_exists('ucssp', $client))
                                {
                                    $ucsInfo = $client['ucssp'];
                                    foreach ($ucsInfo as $ucs)
                                    {
                                        $temp_ucs_array = array();
                                        $temp_ucs_array['instance_name'] = array_key_exists('instance_name', $ucs) ? $ucs['instance_name'] : SystemCapacityReport::UNKNOWN_ELEMENT;
                                        $temp_ucs_array['name'] = $temp_ucs_array['instance_name'];
                                        $temp_ucs_array['last_full'] = $ucs['last_full'];
                                        $temp_ucs_array['host_name'] = $client['name'];
                                        
                                        $temp_system_report[Constants::APPLICATION_TYPE_NAME_UCS_SERVICE_PROFILE][] = $temp_ucs_array;
                                        $report_count += 1;
                                    }
                                }

                                if (array_key_exists('ndmp', $client))
                                {
                                    $ndmpInfo = $client['ndmp'];
                                    foreach ($ndmpInfo as $ndmp)
                                    {
                                        $temp_ndmp_array = array();
                                        $temp_ndmp_array['instance_name'] = array_key_exists('instance_name', $ndmp) ? $ndmp['instance_name'] : SystemCapacityReport::UNKNOWN_ELEMENT;
                                        $temp_ndmp_array['name'] = $temp_ndmp_array['instance_name'];
                                        $temp_ndmp_array['last_full'] = $ndmp['last_full'];
                                        $temp_ndmp_array['host_name'] = $client['name'];
                                        
                                        $temp_system_report[Constants::APPLICATION_TYPE_NAME_NDMP_DEVICE][] = $temp_ndmp_array;
                                        $report_count += 1;
                                    }
                                }

                                $temp_system_report['clients'][] = $temp_client_array;
                                $report_count += 1;
                            }
                        }

                        if( array_key_exists('hyperv', $client) and count($client['hyperv']) > 0 )
                        {
                            foreach ($client['hyperv'] as $hyperv)
                            {
                                $temp_system_report['vms'][] = array( 'hypervisor' => "Hyper-V",
                                    'host_name' => $client['name'],
                                    'name' => $hyperv['vm_name'],
                                    'last_full' => $hyperv['last_full'] );
                                $report_count += 1;
                            }
                        }

                        if( array_key_exists('vmware', $client) and count($client['vmware']) > 0 )
                        {
                            foreach ($client['vmware'] as $vmware)
                            {
                                $temp_system_report['vms'][] = array( 'hypervisor' => "VMware",
                                    'host_name' => $vmware['esx_name'],
                                    'name' => $vmware['vm_name'],
                                    'last_full' => $vmware['last_full'] );
                                $report_count += 1;
                            }
                        }

                        if( array_key_exists('xen', $client) and count($client['xen']) > 0 )
                        {
                            foreach ($client['xen'] as $xen)
                            {
                                $temp_system_report['vms'][] = array( 'hypervisor' => "Xen",
                                    'host_name' => $client['name'],
                                    'name' => $xen['instance_name'],
                                    'last_full' => $xen['last_full'] );
                                $report_count += 1;
                            }
                        }
                    }
                }
                $report_data[] = $temp_system_report;
            }
            else
            {
                if ( count($systems) === 1 )
                {
                    return false;
                }
            }
        }
        $report_total = $report_count;  //Temporary

        $report['count'] = $report_count;
        $report['total'] = $report_total;
        $report['start'] = $report_start;
        $report['data'] = $report_data;
        return $report;
    }

}